﻿using Dalamud.IoC;
using Dalamud.Plugin;
using Dalamud.Plugin.Services;
using Screenie.Ui;

namespace Screenie;

public sealed class Plugin : IDalamudPlugin {
    internal const string Name = "Screenie";

    [PluginService]
    #pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
    internal static IPluginLog Log { get; private set; }
    #pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.

    [PluginService]
    internal DalamudPluginInterface Interface { get; init; }

    [PluginService]
    internal IChatGui ChatGui { get; init; }

    [PluginService]
    internal IClientState ClientState { get; init; }

    [PluginService]
    internal ICommandManager CommandManager { get; init; }

    [PluginService]
    internal IDataManager DataManager { get; init; }

    [PluginService]
    internal IFramework Framework { get; init; }

    [PluginService]
    internal IGameGui GameGui { get; init; }

    [PluginService]
    internal IObjectTable ObjectTable { get; init; }

    internal Configuration Config { get; }
    internal PenumbraIpc Penumbra { get; }
    internal Database Database { get; }
    internal LinkHandlers LinkHandlers { get; }
    internal PluginUi Ui { get; }

    private Command Command { get; }

    #pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
    public Plugin() {
        this.Config = this.Interface!.GetPluginConfig() as Configuration ?? new Configuration();
        this.Penumbra = new PenumbraIpc(this);
        this.Database = new Database(this);
        this.LinkHandlers = new LinkHandlers(this);
        this.Ui = new PluginUi(this);
        this.Command = new Command(this);
    }
    #pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.

    public void Dispose() {
        this.Command.Dispose();
        this.Ui.Dispose();
        this.LinkHandlers.Dispose();
        this.Database.Dispose();
        // this.Penumbra.Dispose();
    }

    internal void SaveConfig() {
        this.Interface.SavePluginConfig(this.Config);
    }
}
